.xmap.internals <- new.env(hash=TRUE,parent=emptyenv())



read.exon <- function(covdesc="covdesc",path=".",...) {
  samples <- read.AnnotatedDataFrame( paste(path,covdesc,sep="/"),sep="");
  files.to.read <- rownames(pData(samples));
  files.to.read <- paste(path,files.to.read,sep="/")
  eset <- ReadAffy(filenames=files.to.read,...);
  newPhenoData <- cbind(pData(eset),pData(samples)[rownames(pData(eset)),]);
  colnames(newPhenoData) <- c(colnames(pData(eset)),colnames(pData(samples)));
  tmp <- as.list(colnames(newPhenoData));
  names(tmp) <- colnames(newPhenoData);
  newPhenoData <- as(newPhenoData,"AnnotatedDataFrame")
  
  phenoData(eset) <- newPhenoData;	
  return(eset);
}



.xmap.load.config <- function() {
    conf.dir <- Sys.getenv("R_XMAP_CONF_DIR")
    if(conf.dir=="") { 
        conf.dir <- file.path(Sys.getenv("HOME"),".exonmap")
    	warning(paste("Environment 'R_XMAP_CONF_DIR' not set. Please refer to INSTALL.TXT for information on how to set this up.\n\nTrying '.exonmap'.\n",sep=""))
    }
    if(!file.exists(conf.dir)) { 
    	stop(paste("Can't find config directory. Please refer to INSTALL.TXT for information on how to set this up.",sep=""))
    }
    .xmap.internals[["conf.dir"]] <- conf.dir
}

.xmap.init <- function() {
  fn <- system.file("mapping.calls",package="exonmap")
  .xmap.call.table <- read.table(fn,header=TRUE)
  .xmap.internals[["calls"]] <- .xmap.call.table 
  .xmap.internals[["use.db.local"]] <- FALSE
  .xmap.internals[["connected"]] <- FALSE
  .xmap.internals[["old.db.warned"]] <- FALSE
}


.get.user.string <-function(v,msg) {
if(v!="<ASK>") {
   r <- v
 }
 else {
   cat(paste(msg,":",sep=""))
   r <- readLines(n=1)
 }
 r
}

xmapConnect <- function(name,username,password) {
  xmapDisconnect()
  if(is.null(.xmap.internals[["conf.dir"]])) .xmap.load.config()
  f <- .xmap.internals[["conf.dir"]]
  f <- file.path(f,"databases.txt")

  if(!file.exists(f)) {
  	stop(paste("Can't find database config file '",f,"'. See package installation instructions for help setting this up.",sep=""))
  }
  a <- read.table(f,sep="\t",strip.white=TRUE,header=TRUE)

  if(!all(colnames(a) == c("name","species","db.name","type","host","port","username","password"))) {
    stop(paste("Unexpected column names in",f," - should be 'name','species','db.name','type','host','port','username' and 'password'"))
  }

  if(missing(name)){
    keep    <- as.character(a$type) == "core"
    choices <-paste(as.character(a[keep,1])," ('",as.character(a[keep,3]),"')",sep="")
     r <- menu(choices,title="Select a database to connect to:")
    if(r == 0) { return(invisible()) }
    else {
      name <- a[keep,1][r]
    }
  }

  dbs <- a[(as.character(a$name) == name),,drop=FALSE]
  if(nrow(dbs) == 0) { stop(paste("No database called '",name,"' found. Try 'xmapConnect()' for a list of available databases.",sep="")) }

  if(missing(username)) {
    un <- .get.user.string(as.character(dbs[dbs$type=="core","username"]),"username")
  }
  else { un <- username }
  if(missing(password)) {
    pw <- .get.user.string(as.character(dbs[dbs$type=="core","password"]),"password")
  }
  else { pw <- password }

  cons <- apply(dbs,1,function(a) {
    port <- as.integer(a[6])
    if(is.na(port)) {
      con <- dbConnect("MySQL",user=un, password=pw, dbname=a[3], host=a[5], client.flag=196608)
    }
    else{
      con <- dbConnect("MySQL",user=un, password=pw, dbname=a[3], host=a[5], port=port, client.flag=196608)
    }
    })
  names(cons) <- dbs[,"type"]
  species <- dbs[,"species"]
  names(species) <- dbs[,"type"]
  db.names <- dbs[,"db.name"]
  names(db.names) <- dbs[,"type"]

  .xmap.internals[["cons"]]     <- cons
  .xmap.internals[["species"]]  <- species
  .xmap.internals[["db.names"]] <- db.names
  .xmap.internals[["connected"]] <- TRUE
   path <- file.path(.xmap.get.conf.dir(),"db.local")
  .xmap.internals[["use.db.local"]] <- FALSE
  fe <- file.exists(path)
  if(!fe) {
     r <- dir.create(path)
     if(!r) {
       .xmap.internals[["use.db.local"]] <- FALSE
     }
     else {
       .xmap.internals[["use.db.local"]] <- .xmap.all.stats.query.exists()
     }
  }
  else { 
    .xmap.internals[["use.db.local"]] <- .xmap.all.stats.query.exists()
  }
  .xmap.internals[["old.db.warned"]] <- FALSE
}

xmapDatabase <- function(name,username,password) { xmapConnect(name,username,password) }

xmapDisconnect <- function() {
  if(!is.null(.xmap.internals[["cons"]])) {
    lapply(.xmap.internals[["cons"]],dbDisconnect)
    remove("cons",envir=.xmap.internals)
    remove("species",envir=.xmap.internals)
    remove("db.names",envir=.xmap.internals)
    if(!is.null(.xmap.internals[["stats.cache"]])) {
      remove("stats.cache",envir=.xmap.internals)
    }
  }
  .xmap.internals[["connected"]] <- FALSE
}

.xmap.get.con <- function(name) {
  if(!.xmap.internals[["connected"]]) { stop("Not connected to database. Use 'xmapConnect()' first.")}
  .xmap.internals[["cons"]][[name]]
}

.xmap.get.species <- function(name) {
  .xmap.internals[["species"]][[name]]
}

.xmap.get.db.name <- function(name) {
  .xmap.internals[["db.names"]][[name]]
}

.xmap.get.conf.dir <- function(name) {
  .xmap.internals[["conf.dir"]]
}

.xmap.use.db.local <- function() {
  .xmap.internals[["use.db.local"]]
}

.xmap.recent.db <- function() {
  .xmap.use.db.local()
}

db.local.info <- function() {
  f <- file.path(.xmap.get.conf.dir(),"db.local")
  if(file.exists(f)) {
    cat(paste("Local directory is '",f,"'.\n",sep=""))
    cat("Contents:\n")
    cat(paste(list.files(f),collapse="\n"))
    cat("\n")
  }
  else {
    cat("db.local directory not set up.")
  }
}


clear.db.local <- function() {
  f <- file.path(.xmap.get.conf.dir(),"db.local")
  if(file.exists(f)) {
    file.remove(file.path(f,list.files(f)))
  }
  return(invisible())
}

.xmap.all.stats.query.exists <- function() {
  tryCatch(
           {
             sql <- "xmap_allProbesets"
             con <- .xmap.get.con("core")
             r   <- .xmap.call(v=NULL,sql=sql,con=con)
             return(TRUE)
           },
           error=function(e) { return(FALSE) }
           )
}

.xmap.call.map <- function(n,subset) {
  m <- .xmap.internals[["calls"]]
  r <- m[(m$name==n) & (m$subset==subset),]
  if(dim(r)[1]==0) { 
    stop(paste("Mapping via the '",subset,"' subset not supported.",sep=""))
  }
  r
}



.exit.cleanup <- function(connection) {
  res <-  dbListResults(connection)
  for(i in res)  {
    if(!dbHasCompleted(i)) {
      fetch(i,-1)
      dbClearResult(i)
    }
  }
  while(dbMoreResults(connection)) {
    res <- dbNextResult(connection)
    if(!dbHasCompleted(res)) {
      fetch(res,-1)
      dbClearResult(res)
    }
  }
}


.xmap.raw.call <- function(raw.sql,con) {
  on.exit(.exit.cleanup(con))
  res  <- dbSendQuery( con, raw.sql )
  d    <- fetch(res,-1)
  while(dbMoreResults(con)) {
    res <- dbNextResult(con)
    if(!dbHasCompleted(res)) {
      fetch(res,-1)
    }
  }
  dbClearResult(res)
  d
}


.xmap.call <- function(v,sql,con,is.integer=FALSE) {
  if(is.null(v)) {
    sql <- paste("CALL ",sql,"()",sep="")
  }
  else if(is.integer) {
    sql <- paste("CALL ",sql,"(",v,")",sep="")
  }
  else {
    sql <- paste("CALL ",sql,"('",v,"')",sep="")
  }
  .xmap.raw.call(sql,con)
}


.xmap.in.range.call <- function(start,stop,strand,chr,call,con) {
  sql <- paste("CALL ",call,"(",start,",",stop,",",strand,",'",chr,"')",sep="")
  .xmap.raw.call(sql,con)
}



.old.db.warn <- function() {
  if(!.xmap.internals[["old.db.warned"]]) {
    warning("You are using an old version of X:Map. This will not be compatible with future versions of exonmap; you are advised to upgrade to a more recent version of the database as soon as possible.")
    .xmap.internals[["old.db.warned"]] <- TRUE
  }
}
