.build.sc <- function() {
  sql <- "xmap_allProbesets"
  con <- .xmap.get.con("core")
  r   <- .xmap.call(v=NULL,sql=sql,con=con)
  bits <-     ((r[,3] == 1) & (r[,4] >  0) & (r[,5] >  0)) +
          2 * ((r[,3] == 1) & (r[,4] == 0) & (r[,5] >  0)) +
          4 * ((r[,3] == 1) & (r[,4] == 0) & (r[,5] == 0)) +
          8 *  (r[,3] > 1)
  names(bits) <- r[,2]
  .xmap.internals[["stats.cache"]] <- bits
  
  fn   <- paste("stats.core.",.xmap.get.db.name("core"),".R",sep="")
  path <- file.path(.xmap.get.conf.dir(),"db.local")
  if(!file.exists(path)) {
  	dir.create(path)
  }
  path <- file.path(.xmap.get.conf.dir(),"db.local",fn)
  save("stats.cache",envir=.xmap.internals,file=path)
  bits
}


.load.sc <- function() {
  fn   <- paste("stats.core.",.xmap.internals[["db.names"]]["core"],".R",sep="")
  path <- file.path(.xmap.internals[["conf.dir"]],"db.local",fn)
  if(!file.exists(path)) {
  	r <- NULL
  }
  else {
  	load(path,envir=.xmap.internals)
  	r <- .xmap.internals[["stats.cache"]]
  }
  r
}

.xmap.get.stats.cache <- function() {
  sc <- .xmap.internals[["stats.cache"]]
  if(is.null(sc)) {
    sc <- .load.sc()
  }
  if(is.null(sc)) {
    sc <- .build.sc()
  }
  sc
}

.is.exonic.c <- function(v) {
  r <- .xmap.get.stats.cache()
  v <- intersect(v,names(r))
  r <- (r[v] == 1)
  names(r) <- v
  r
}


.is.intronic.c <- function(v) {
  r <- .xmap.get.stats.cache()
  v <- intersect(v,names(r))
  r <- (r[v] == 2)
  names(r) <- v
  r
}


.is.intergenic.c <- function(v) {
  r <- .xmap.get.stats.cache()
  v <- intersect(v,names(r))
  r <- (r[v] == 4)
  names(r) <- v
  r
}

.is.multitarget.c <- function(v) {
  r <- .xmap.get.stats.cache()
  v <- intersect(v,names(r))
  r <- (r[v] >= 8)
  names(r) <- v
  r
}

.exonic.c <- function(v,exclude=FALSE,mt.rm=TRUE) {
  r <- .xmap.get.stats.cache()
  v <- intersect(v,names(r))
  r <- r[v]
  if(mt.rm) {
    r <- names(r)[xor(r==1,exclude)]
  }
  else {
    r <- names(r)[xor(r==9,exclude)]
  }
  
  r
}

.intronic.c <- function(v,exclude=FALSE,mt.rm=TRUE) {
  r    <- .xmap.get.stats.cache()
  v <- intersect(v,names(r))
  r <- r[v]
  if(mt.rm) {
    r <- names(r)[xor(r==2,exclude)]
  }
  else {
    r <- names(r)[xor(r==10,exclude)]
  }
  r
}


.intergenic.c <- function(v,exclude=FALSE,mt.rm=TRUE) {
  r    <- .xmap.get.stats.cache()
  v <- intersect(v,names(r))
  r <- r[v]
  if(mt.rm) {
    r <- names(r)[xor(r==4,exclude)]
  }
  else {
    r <- names(r)[xor(r==12,exclude)]
  }
  r
}


.multitarget.c <- function(v,exclude=FALSE) {
  r    <- .xmap.get.stats.cache()
  v <- intersect(v,names(r))
  r <- r[v]
  r <- names(r)[xor(r>=8,exclude)]
  r
}


probeset.stats <- function(v) {
  if(length(v) == 0) { return(NULL) }
  v   <- .make.params(v)
  sql <- "xmap_probesetSummary"
  con <- .xmap.get.con("core")
  r   <- lapply(v,.xmap.call,sql=sql,con=con)
  r   <- do.call(rbind,r)
  r
}

is.exonic <- function(v) {
  if(.xmap.use.db.local()) return(.is.exonic.c(v))
  r <- probeset.stats(v)
  if(any(colnames(r) == "probeset")) { psn <- "probeset" } else { psn <- "probeset_name" }
  r <- r[(r[,"hitScore"] == 1) & (r[,"exonScore"] > 0) & (r[,"geneScore"] > 0),psn]
  v %in% r
}

is.intronic <- function(v) {
  if(.xmap.use.db.local()) return(.is.intronic.c(v))
  r <- probeset.stats(v)
  if(any(colnames(r) == "probeset")) { psn <- "probeset" } else { psn <- "probeset_name" }
  r <- r[(r[,"hitScore"] == 1) & (r[,"exonScore"] == 0) & (r[,"geneScore"] > 0),psn]
  v %in% r
}

is.intergenic <- function(v) {
  if(.xmap.use.db.local()) return(.is.intergenic.c(v))
  r <- probeset.stats(v)
  if(any(colnames(r) == "probeset")) { psn <- "probeset" } else { psn <- "probeset_name" }
  r <- r[(r[,"hitScore"] == 1) & (r[,"exonScore"] == 0) & (r[,"geneScore"] == 0),psn]
  v %in% r
}

is.multitarget <- function(v,mt.level=1) {
  if(!(.xmap.use.db.local()) & (mt.level ==1 )) return(.is.multitarget.c(v))
  r <- probeset.stats(v)
  if(any(colnames(r) == "probeset")) { psn <- "probeset" } else { psn <- "probeset_name" }

  r <- r[r[,"hitScore"]>mt.level,psn]
  v %in% r
}

exonic <- function(v,exclude=FALSE,mt.rm=TRUE) {
  if(.xmap.use.db.local()) return(.exonic.c(v,exclude,mt.rm))
  r    <- probeset.stats(v)
  if(any(colnames(r) == "probeset")) { psn <- "probeset" } else { psn <- "probeset_name" }
  if(mt.rm) {
    keep <- xor((r[,"exonScore"] > 0) & (r[,"geneScore"] > 0) & (r[,"hitScore"] == 1),exclude)
  }
  else {
    keep <- xor((r[,"exonScore"] > 0) & (r[,"geneScore"] > 0),exclude)
  }
  (r[keep,psn])
}


intronic <- function(v,exclude=FALSE,mt.rm=TRUE) {
  if(.xmap.use.db.local()) return(.intronic.c(v,exclude,mt.rm))
  r    <- probeset.stats(v)
  if(any(colnames(r) == "probeset")) { psn <- "probeset" } else { psn <- "probeset_name" }
  if(mt.rm) {
    keep <- xor((r[,"exonScore"] == 0) & (r[,"geneScore"] > 0) & (r[,"hitScore"] == 1),exclude)
  }
  else {
    keep <- xor((r[,"exonScore"] == 0) & (r[,"geneScore"] > 0),exclude)
  }
  r[keep,psn]
}

intergenic <- function(v,exclude=FALSE,mt.rm=TRUE) {
  if(.xmap.use.db.local()) return(.intergenic.c(v,exclude,mt.rm))
  r    <- probeset.stats(v)
  if(any(colnames(r) == "probeset")) { psn <- "probeset" } else { psn <- "probeset_name" }
  if(mt.rm) {
    keep <- xor((r[,"exonScore"] == 0) & (r[,"geneScore"] == 0) & (r[,"hitScore"] == 1),exclude)
  }
  else {
    keep <- xor((r[,"exonScore"] == 0) & (r[,"geneScore"] == 0),exclude)
  }
  r[keep,psn]
}

multitarget <- function(v,exclude=FALSE,mt.level=1) {
  if(.xmap.use.db.local() & (mt.level == 1)) return(.multitarget.c(v,exclude))
  r     <- probeset.stats(v)
  if(any(colnames(r) == "probeset")) { psn <- "probeset" } else { psn <- "probeset_name" }
  keep  <- xor((r[,"hitScore"] > mt.level),exclude)
  r[keep,psn]
}

select.probewise <- function(v,filter=c("exonic","intronic","intergenic","multitarget"),mt.rm=TRUE,mt.level=1) {
  filter <- match.arg(filter)
  switch(filter,
         exonic=exonic(v,exclude=FALSE,mt.rm),
         intronic=intronic(v,exclude=FALSE,mt.rm),
         intergenic=intergenic(v,exclude=FALSE,mt.rm),
         multitarget=multitarget(v,exclude=FALSE,mt.level))
}

exclude.probewise <- function(v,filter=c("exonic","intronic","intergenic","multitarget"),mt.rm=TRUE,mt.level=1) {
  filter <- match.arg(filter)
  switch(filter,
         exonic=exonic(v,exclude=TRUE,mt.rm),
         intronic=intronic(v,exclude=TRUE,mt.rm),
         intergenic=intergenic(v,exclude=TRUE,mt.rm),
         multitarget=multitarget(v,exclude=TRUE,mt.level))
}

