\name{filters}
\alias{filters}
\alias{exonic}
\alias{intronic}
\alias{intergenic}
\alias{multitarget}
\alias{is.exonic}
\alias{is.intronic}
\alias{is.intergenic}
\alias{is.multitarget}
\alias{select.probewise}
\alias{exclude.probewise}



\title{Filter vectors of Affymetrix Exon array probeset names
  according to where they target}
\description{
  Given a character vector of probeset names, filter it to keep (or
  exclude) those probesets that are mapped by X:Map to exons, introns,
  intergenic regions or are annotated as containing potentially
  cross-hybridizing (multitarget) probes. Functions of the form
  \code{is. ...} return a logical vector.
}
\usage{
  exonic(v,exclude=FALSE,mt.rm=TRUE)
  intronic(v,exclude=FALSE,mt.rm=TRUE)
  intergenic(v,exclude=FALSE,mt.rm=TRUE)
  multitarget(v,exclude=FALSE,mt.level=1)
  is.exonic(v)
  is.intronic(v)
  is.intergenic(v)
  is.multitarget(v,mt.level=1)
  select.probewise(v,filter=c("exonic","intronic","intergenic","multitarget"),mt.rm=TRUE,mt.level=1)
  exclude.probewise(v,filter=c("exonic","intronic","intergenic","multitarget"),mt.rm=TRUE,mt.level=1)

}
\arguments{
 \item{v}{A character vector of probeset names identifiers}
 \item{exclude}{If TRUE, remove (rather than keep) matching probesets, from the list.}
 \item{mt.rm}{If TRUE, remove multitarget probesets before returning the
   result}
 \item{mt.level}{The amount of multitargeting needed before a probeset
   is removed. See the package vignette for more details on how this
   number is calculated.}
 \item{filter}{What sort of probeset should be retained/removed?}
}

\details{
  Connects to the X:Map database to retrieve data. Before these
  functions can be used, \code{\link{xmapConnect}} must have been
  called.

  By default, multitarget probesets are removed.
}
\value{
A character vector of filtered names.
}
\seealso{
  \link{xmapConnect} \link{mappings} \link{details}
}
\examples{
  if(interactive()) {
    xmapConnect()
    gene <-  probeset.to.gene(c("3743919"))
    ps   <- gene.to.probeset(gene,as.vector=TRUE)
    exonic(ps)
    intronic(ps)
    intergenic(ps)
    multitarget(ps)
    exonic(ps,exclude=TRUE)
    intronic(ps,exclude=TRUE)
    intergenic(ps,exclude=TRUE)
    multitarget(ps,exclude=TRUE,mt.level=1)
    #or
    select.probewise(ps,"exonic") #etc..
    exclude.probewise(ps,"exonic") #etc..
    is.exonic(ps)
    is.intronic(ps)
    is.intergenic(ps)
    is.multitarget(ps)
    is.multitarget(ps,mt.level=4)

  }
}
\keyword{misc}
\author{C.J. Miller, M.J. Okoniewski}
\references{\url{http://xmap.picr.man.ac.uk}}
